@@ -15,6 +15,16 @@ class SCIMObject(object):
             setattr(self, key, kwargs[key])
 
     def to_json(self):
+        if hasattr(self, 'APPARTENANCEORG_SCHEMA') and hasattr(self, 'APPARTENANCEORG_ATTRS'):
+            json_dict = json.loads(json.dumps(self, default=lambda o: o.__dict__))
+            app_org_ready_json = {}            
+            for key in json_dict:
+                if key == self.APPARTENANCEORG_ATTRS[0]:
+                    app_org_ready_json[self.APPARTENANCEORG_SCHEMA] = {key: json_dict[key].copy()}
+                else:
+                    app_org_ready_json[key] = json_dict[key]
+            return json.dumps(app_org_ready_json)
+
         return json.dumps(self, default=lambda o: o.__dict__)
 
     @classmethod
@@ -47,7 +57,7 @@ class User(SCIMObject):
     USERSTATE_ATTRS = ['lastSuccessfulLoginDate', 'lastFailedLoginDate',
     'loginAttempts', 'locked']
     
-    APPARTENANCEORG_ATTRS = ["urn:sadupanorama:scim:api:messages:2.0:AppartenancesOrganisationnelles"]
+    APPARTENANCEORG_ATTRS = ["appartenancesOrganisationnelles"]
 
     def __init__(self, *initial_data, **kwargs):
         self.schemas = [User.CORE_USER_SCHEMA]
@@ -56,28 +66,30 @@ class User(SCIMObject):
 
         for dictionary in initial_data:
             for key in dictionary:
-                if key in User.APPARTENANCEORG_ATTRS:
-                    for appOrgKey in dictionary[key]:
-                        setattr(self, appOrgKey, dictionary[key][appOrgKey])
+                if key == User.APPARTENANCEORG_SCHEMA:
+                    for app_org_key in dictionary[key]:
+                        if app_org_key in User.APPARTENANCEORG_ATTRS:
+                            setattr(self, app_org_key, dictionary[key][app_org_key])
                 else:
                     setattr(self, key, dictionary[key])
         for key in kwargs:
-            if key in User.APPARTENANCEORG_ATTRS:
-                for appOrgKey in kwargs[key]:
-                    setattr(self, appOrgKey, kwargs[key][appOrgKey])
+            if key == User.APPARTENANCEORG_SCHEMA:
+                for app_org_key in kwargs[key]:
+                    if app_org_key in User.APPARTENANCEORG_ATTRS:
+                        setattr(self, app_org_key, kwargs[key][app_org_key])
             else:
                 setattr(self, key, kwargs[key])
 
     def __setattr__(self, name, value):
-        if name in User.ENTERPRISE_ATTRS:
+        if name in User.ENTERPRISE_ATTRS and User.ENTERPRISE_ATTRS not in self.schemas:
             self.schemas += [User.ENTERPRISE_USER_SCHEMA]
-        elif name in User.IDCS_ATTRS:
+        elif name in User.IDCS_ATTRS and User.IDCS_ATTRS not in self.schemas:
             self.schemas += [User.IDCS_USER_SCHEMA]
-        elif name in User.PASSWORDSTATE_ATTRS:
+        elif name in User.PASSWORDSTATE_ATTRS and User.PASSWORDSTATE_ATTRS not in self.schemas:
             self.schemas += [User.PASSWORDSTATE_USER_SCHEMA]
-        elif name in User.USERSTATE_ATTRS:
+        elif name in User.USERSTATE_ATTRS and User.USERSTATE_ATTRS not in self.schemas:
             self.schemas += [User.USERSTATE_USER_SCHEMA]
-        elif name in User.APPARTENANCEORG_ATTRS:
+        elif name in User.APPARTENANCEORG_ATTRS and User.APPARTENANCEORG_SCHEMA not in self.schemas:
             self.schemas += [User.APPARTENANCEORG_SCHEMA]
             
         self.__dict__[name] = value
@@ -89,7 +101,7 @@ class User(SCIMObject):
         updated.update(actual)
         updated.update(updates)
         return self.from_json(json.dumps(updated))
-    
+
 class Group(SCIMObject):
     URI = "/Groups"
 
@@ -134,76 +146,77 @@ class SCIMClient(object):
             self.headers["Authorization"] = "bearer " + access_token
         if additionnal_headers is not None and len(additionnal_headers.keys()) > 0:
             for header in additionnal_headers.keys():
-                self.headers[header] = additionnal_headers[header]
+                if header.upper() != 'CONTENT-TYPE':
+                    self.headers[header] = additionnal_headers[header]
         
     def searchUserByUserName(self, userName):
-        userSearchUrl = self.base_url + User.URI + '/.search'
+        user_search_url = self.base_url + User.URI + '/.search'
         data = '{"filter":"userName eq \\\"' + userName + '\\\""}"'
         try:
-            response = requests.post(url=userSearchUrl, headers=self.headers, data=data)
+            response = requests.post(url=user_search_url, headers=self.headers, data=data)
             users = response.json()
             if response.status_code != 200:
                 raise SCIMException(
-                'Could not search for user %s at %s: %s' % (userName, userSearchUrl, str(response.status_code)))
+                'Could not search for user %s at %s: %s' % (userName, user_search_url, str(response.status_code)))
             if "Resources" in users and len(users["Resources"]) > 0:
                 return User.from_json(json.dumps(users["Resources"][0]))
             return None
         except Exception as e:
             raise SCIMException(
-                'Could not search for user %s at %s: %s' % (userName, userSearchUrl, str(e)))
+                'Could not search for user %s at %s: %s' % (userName, user_search_url, str(e)))
         
     def getMe(self):
-        meUrl = self.base_url + '/Me'
+        me_url = self.base_url + '/Me'
         try:
-            response = requests.get(url=meUrl, headers=self.headers)
+            response = requests.get(url=me_url, headers=self.headers)
             if response.status_code == 200: 
                 return User.from_json(response.content)
-            raise SCIMException('Could not get Me at %s: %s' % (meUrl, response.json()))
+            raise SCIMException('Could not get Me at %s: %s' % (me_url, response.json()))
         except Exception as e:
-            raise SCIMException('Could not get Me at %s: %s' % (meUrl, str(e)))
+            raise SCIMException('Could not get Me at %s: %s' % (me_url, str(e)))
 
     def getUserById(self, id):
-        userUrl = self.base_url + User.URI + '/' + id
+        user_url = self.base_url + User.URI + '/' + id
         try:
-            response = requests.get(url=userUrl, headers=self.headers)
+            response = requests.get(url=user_url, headers=self.headers)
             if response.status_code == 200: 
                 return User.from_json(response.content)
-            raise SCIMException('Could not get user id %s at %s: %s' % (id, userUrl, response.json()))
+            raise SCIMException('Could not get user id %s at %s: %s' % (id, user_url, response.json()))
         except Exception as e:
-            raise SCIMException('Could not get user id %s at %s: %s' % (id, userUrl, str(e)))
+            raise SCIMException('Could not get user id %s at %s: %s' % (id, user_url, str(e)))
 
     def createUser(self, user):
-        userUrl = self.base_url + User.URI
+        user_url = self.base_url + User.URI
         try:
             if "externalId" not in json.loads(user.to_json()):
                 user.externalId = str(uuid.uuid1())
             data = user.to_json()
-            response = requests.post(url=userUrl, headers=self.headers, data=data)
+            response = requests.post(url=user_url, headers=self.headers, data=data)
             if response.status_code == 201 or response.status_code == 200: 
                 return User.from_json(response.content)
-            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, userUrl, response.json()))        
+            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, user_url, response.json()))        
         except Exception as e:
-            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, userUrl, str(e)))
+            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, user_url, str(e)))
     
     
     def deleteUser(self, user):
-        userUrl = self.base_url + User.URI + '/' + user.id
+        user_url = self.base_url + User.URI + '/' + user.id
         try:
-            response = requests.delete(url=userUrl, headers=self.headers)
+            response = requests.delete(url=user_url, headers=self.headers)
             return response
         except Exception as e:
-            raise SCIMException('Could not delete user %s at %s: %s' % (user.userName, userUrl, str(e)))
+            raise SCIMException('Could not delete user %s at %s: %s' % (user.userName, user_url, str(e)))
         
     def updateUser(self, user):
-        userUrl = self.base_url + User.URI + '/' + user.id
+        user_url = self.base_url + User.URI + '/' + user.id
         try:
             data = user.to_json()
-            response = requests.put(url=userUrl, headers=self.headers, data=data)
+            response = requests.put(url=user_url, headers=self.headers, data=data)
             if response.status_code == 200: 
                 return User.from_json(response.content)
-            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, userUrl, response.json()))
+            raise SCIMException('Could not create user %s at %s: %s' % (user.userName, user_url, response.json()))
         except Exception as e:
-            raise SCIMException('Could not update user %s at %s: %s' % (user.userName, userUrl, str(e)))
+            raise SCIMException('Could not update user %s at %s: %s' % (user.userName, user_url, str(e)))
             
 class SCIMException(Exception):
     """ Exception SCIM """
